#version 430 core
/*
* Copyright 2018 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
        vec4 FragPosLightSpace; //shadowMap
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;

uniform sampler2DShadow	shadowMap; // shadowMap

uniform float brightness;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

const float levels = 3.0f;


float PCF(vec3 projCoords, float currentDepth, float bias){
    float shadow = 0.0;
    //returns a vec2 of the width and height of the given sampler texture at mipmap level 0
    //Divided over 1 returns size of a single texel used to offset the texture coordinates
    //sample 9 values around projected coordinate's x and y value, test for shadow occlusion 
    //average the results by the total number of samples taken.
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0); 
   
    for(int i = -1; i <= 1; ++i)
    {   
        for(int j = -1; j <= 1; ++j)
         {
            //float pcfDepth = texture(shadowMap, projCoords.xy + vec2(i,j) * texelSize).r;
            float pcfDepth = texture(shadowMap, vec3((projCoords.xy + vec2(i,j) * texelSize),projCoords.z-bias)).r;//+0.001)).r;

            shadow += (currentDepth - bias) <= pcfDepth ? 1.0 : 0.0;
          }
    }
    shadow /= 9.0;
    return shadow;
}

float ShadowCalculation(vec4 fragPosLightSpace, vec3 lightDir, vec3 n)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;
    float shadow;

    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // check whether current frag pos is in shadow

    float bias = max(0.05 * (1.0 - dot(n, lightDir)), 0.005); // for shadow acne
    //float bias = 0.005;

    float closestDepth = texture(shadowMap, vec3(projCoords.xy,projCoords.z-bias)).r;//+0.001)).r; 
    //float closestDepth = texture(shadowMap, projCoords.xy).r; 
    
    //float shadow = currentDepth <= closestDepth  ? 1.0 : 0.0; //shadow=1.0 no shadow    
    shadow = (currentDepth - bias) <= closestDepth  ? 1.0 : 0.0; //shadow=1.0 no shadow
    
    shadow = PCF(projCoords, currentDepth, bias);
     
    if(projCoords.z > 1.0f)shadow = 1.0f; //to add
    return shadow; 
}

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;
       
        float colorBrightness = max(0,dot(n,l))*brightness;	
	float level = floor(colorBrightness * levels);
        colorBrightness = level / levels; 	
		
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	//return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
        return (diffuseF * diffuseC * colorBrightness + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att;

}

void main() {	

        vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	//color = vec4(texColor * materialCoefficients.x, 1); // ambient
        if(brightness >= 0.6)color = vec4(texColor * 0.45,1); // ambient
        else color = vec4(texColor * brightness,1);
   
        float shadow = ShadowCalculation(vert.FragPosLightSpace, dirL.direction, n);  //shadowMap

	// add directional light contribution
	color.rgb += shadow * phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
        			
	// add point light contribution
	/*color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);*/

}

